from pathlib import Path

from cognite.airworkflow.model.project import Project, ProjectInfo
from cognite.airworkflow.util import env, repoconfighelpers

# List of base constants
working_path = env.get_env_value("PWD")
ROOT_DIR = Path(working_path)
FUNCTIONS_PATH = ROOT_DIR / "functions"
IGNORE_MODELS_PATH = ROOT_DIR / ".ignore_models"

# Paths within a functions folder
FUNCTION_REL_PATH = Path("function")
FUNCTION_REL_CONFIG_PATH = FUNCTION_REL_PATH / "config.yaml"
FUNCTION_REL_RESOURCE_PATH = FUNCTION_REL_PATH / "resources"
FUNCTION_REL_DEPLOYMENT_PATH = FUNCTION_REL_RESOURCE_PATH / "dependencies.yaml"
FUNCTION_REL_INIT_PATH = FUNCTION_REL_PATH / "__init__.py"

# WORK FLOW PATHS
WORK_FLOW_PATH = ROOT_DIR / ".github" / "workflows"
WORK_FLOWS = [WORK_FLOW_PATH / p for p in ["build-master.yaml", "build-pr.yaml", "delete-pr.yaml"]]

# CONFIG PATHS
BASE_PATH = Path(__file__).parent.resolve()
SCHEMAS_PATH = BASE_PATH / "schemas"
DEPLOYMENT_SCHEMA_PATH = SCHEMAS_PATH / "deployment-schema.yaml"
CONFIG_SCHEMA_PATH = SCHEMAS_PATH / "config-schema.yaml"

url_dict = repoconfighelpers.get_url_dict()
if Project.NAME.value in url_dict:
    PROJECT_TO_API_KEYS = {Project.NAME: ProjectInfo(Project.NAME, "AIR_API_KEY", str(url_dict[Project.NAME.value]))}
else:
    PROJECT_TO_API_KEYS = {Project.NAME: ProjectInfo(Project.NAME, "AIR_API_KEY")}



import os
import sys
from itertools import product

import cognite.airworkflow.constants as const
from cognite.airworkflow.function_deployer import FunctionDeployer
from cognite.airworkflow.master_config import get_deployments, get_master_config
from cognite.airworkflow.model.schedule_assets import execute
from cognite.airworkflow.model_asset_hierarchy import ModelAssetHierarchy
from cognite.airworkflow.util import cdf, env, functions, github


def handle_asset_hierarchy():
    _, project_to_config = get_master_config()
    for project, config in project_to_config.items():
        print(f"Running Model Asset Hierarchy create/update for project {project}")
        ModelAssetHierarchy(config, project).create_or_update()


def handle_function_deployments():
    github_event_name = env.get_env_value("GITHUB_EVENT_NAME")
    github_ref = env.get_env_value("GITHUB_REF")
    is_delete = "IS_DELETE" in os.environ and os.environ["IS_DELETE"] == "true"
    is_pr = github_event_name == "pull_request"

    assert (
        len(sys.argv) >= 3 and sys.argv[2]
    ), "Expected a function name passed as a command line argument, but got nothing"
    function_name = sys.argv[2].split("/")[-1] if "/" in sys.argv[2] else sys.argv[2]
    print(f"HANDLING {github_event_name} ON {github_ref} FOR FUNCTION {function_name}")

    for deployment in get_deployments(function_name):
        FunctionDeployer(deployment, is_delete, is_pr).handle_deployment()


def handle_delete_functions():
    """
    1. Find name of all functions that should be deployed: Based on what's on master and what's in PRs
    2. List all functions currently deployed
    3. Delete the diff
    """
    _, project_to_config = get_master_config()
    for project, config in project_to_config.items():
        function_paths = [functions.get_relative_function_path(p) for p in config]
        pr_refs = github.get_open_pr_refs()
        all_functions = [functions.get_function_name(f, latest=True) for f in function_paths] + [
            functions.get_function_name(f, pr=True, ref=ref) for ref, f in list(product(pr_refs, function_paths))
        ]

        info = const.PROJECT_TO_API_KEYS[project]
        client = cdf.experimental_client(info.project)
        dangling = functions.list_dangling_function(client, all_functions, name_prefix=env.get_repo_name_auto())
        for function in dangling:
            functions.delete_function(client, function.external_id)


def handle_deploy_infrastructure_schedules():
    _, project_to_config = get_master_config()
    for project, config_dict in project_to_config.items():
        info = const.PROJECT_TO_API_KEYS[project]
        client = cdf.experimental_client(info.project)
        execute(client)


def handle_call_schedulemanager():
    _, project_to_config = get_master_config()
    for project in project_to_config:
        client = cdf.experimental_client(project)
        client.functions.call(external_id="cognitedata/air-functions/schedulemanager:latest")


if __name__ == "__main__":

    if sys.argv[1] == "model":
        handle_asset_hierarchy()
    elif sys.argv[1] == "function":
        handle_function_deployments()
    elif sys.argv[1] == "delete":
        handle_delete_functions()
    elif sys.argv[1] == "infraschedules":
        handle_deploy_infrastructure_schedules()
    elif sys.argv[1] == "call_schedulemanager":
        handle_call_schedulemanager()



import os
import sys
from pathlib import Path
from typing import Dict

from ruamel.yaml import YAML

import cognite.airworkflow.util.file as file
from cognite.airworkflow.util import env


def load_yaml(path: Path) -> Dict:
    yaml = YAML(typ="safe").load(path)
    assert isinstance(yaml, dict)
    return yaml


working_path = env.get_env_value("PWD")
ROOT_DIR = Path(working_path)
FUNCTIONS_PATH = ROOT_DIR / "functions"
IGNORE_MODELS_PATH = ROOT_DIR / ".ignore_models"

# Paths within a functions folder
FUNCTION_REL_PATH = Path("function")
FUNCTION_REL_CONFIG_PATH = FUNCTION_REL_PATH / "config.yaml"


def project_name_finder() -> str:
    ignore_models = file.read_file_to_list(IGNORE_MODELS_PATH)
    directory_contents = os.listdir(FUNCTIONS_PATH)
    directory_paths = list(
        map(
            lambda x: FUNCTIONS_PATH / x,
            filter(lambda x: x not in ignore_models, directory_contents),
        )
    )
    function_paths = [x / FUNCTION_REL_CONFIG_PATH for x in directory_paths if os.path.isdir(x)]

    if not function_paths:
        raise ValueError("No config files have been defined ! Please define config files for your function !")
        sys.exit(1)
    config_path = function_paths[0]
    yamlload = load_yaml(config_path)
    try:
        project_name = yamlload["modelSettings"]["deploy"]
        if len(project_name) > 1:
            raise ValueError("Please declare only a single project name in the function ")
            sys.exit(1)
        elif len(project_name) == 0:
            raise ValueError("Please declare a project name in the function config file!")
            sys.exit(1)
        else:
            return str(project_name[0])
    except KeyError:
        sys.exit(1)
